﻿-- Reference to Pitbull 4
local _G 		= _G
local PitBull4 	= _G.PitBull4
if not PitBull4 then
	error("PitBull4_ThreatLine requires PitBull4")
end


-- Translations und new module
local Lpitbull 				= PitBull4.L
local Lmodule				= LibStub("AceLocale-3.0"):GetLocale("PitBull4_ThreatLine")
local PitBull4_ThreatLine 	= PitBull4:NewModule("ThreatLine", "AceEvent-3.0")


-- Local variables
local player_in_group = false			-- Is player in a group?
local ACCEPTABLE_CLASSIFICATIONS = {	-- Frames that are allowed to use this module
	player 		= true,
	pet 		= true,
	party 		= true,
	raid 		= true,
	partypet 	= true,
	raidpet 	= true,
	target 		= true,
	focus 		= true,
}
local global_defaults = {
	colors = {						-- Default coloring
		[0] = {						-- Having aggro
			[0] = {r = 0.1, g = 0.4, b = 0.1},
			[1] = {r = 0.1, g = 0.8, b = 0.1},
		},
		[1] = {						-- Other tank aggro
			[0] = {r = 0.1, g = 0.0, b = 0.3},
			[1] = {r = 0.1, g = 0.4, b = 0.7},
		},
		[2] = {						-- Non-tank aggro
			[0] = {r = 0.4, g = 0.1, b = 0.1},	
			[1] = {r = 0.8, g = 0.1, b = 0.1},	
		},
		[3] = {						-- Undefined
			[0] = {r = 0.4, g = 0.4, b = 0.4},	
			[1] = {r = 0.4, g = 0.4, b = 0.4},	
		},
	},
	use_gradient = false,
}
local exclude = {						-- Exclude player (for default max threat holder search)
	["player"] = true,
}
local luaTexts = {
	[Lmodule["Threat Line: Relative player threat - Informational"]] = {
		events 	= {
			["PLAYER_TARGET_CHANGED"] 			= true, 
			["UNIT_THREAT_LIST_UPDATE"] 		= true, 
			["UNIT_THREAT_SITUATION_UPDATE"] 	= true, 
			["PARTY_MEMBERS_CHANGED"] 			= true, 
			["RAID_ROSTER_UPDATE"] 				= true, 
			["PLAYER_PET_CHANGED"] 				= true, 
		},
		code 	= [[-- Query threat, holder and color
			local namespace           = _G.ThreatLine_LuaTexts
			local maxThreat, maxID    = namespace.GetPlayerRelativeThreat(unit)
			local r, g, b             = namespace.GetPlayerTankedColor(unit, (maxThreat or 0) / 100)

			-- Query holder's name
			local name = ""
			if maxID and not UnitIsUnit(maxID, "player") then
				name = UnitName(maxID)
			end

			-- Threat info available
			if maxThreat and maxThreat ~= 0 then
				return "|cff%02x%02x%02x%s%%\n%s|r", r*255, g*255, b*255, Round(maxThreat, 1), name
			end
			return ConfigMode()]]
		,
	},
	[Lmodule["Threat Line: Relative player threat - Simple"]] = {
		events 	= {
			["PLAYER_TARGET_CHANGED"] 			= true, 
			["UNIT_THREAT_LIST_UPDATE"] 		= true, 
			["UNIT_THREAT_SITUATION_UPDATE"] 	= true, 
			["PARTY_MEMBERS_CHANGED"] 			= true, 
			["RAID_ROSTER_UPDATE"]				= true, 
			["PLAYER_PET_CHANGED"] 				= true, 
		},
		code 	= [[-- Query threat, holder and color
			local namespace           = _G.ThreatLine_LuaTexts
			local maxThreat, maxID    = namespace.GetPlayerRelativeThreat(unit)

			-- Threat info available
			if maxThreat and maxThreat ~= 0 then
				return "%s%%", Round(maxThreat, 1)
			end
			return ConfigMode()]]
		,
	},
} 									-- Predefined LuaText


-- Global namespace such that it can be used by LuaText
local ThreatLine_LuaTexts = {}
_G.ThreatLine_LuaTexts = ThreatLine_LuaTexts


-- ThreatLine bar module settings
PitBull4_ThreatLine:SetModuleType("bar")
PitBull4_ThreatLine:SetName(Lmodule["Threat Line"])
PitBull4_ThreatLine:SetDescription(Lmodule["Show a relative threat bar."])
PitBull4_ThreatLine:SetDefaults({
	size 		= 1.0,
	position 	= 6,
}, {
	ThreatLineDB 	= PitBull4.Utils.deep_copy(global_defaults),
})


-- Much like UnitDetailedThreatSituation(...) status but with raid tank support
local function GetTankedStatus(unitID, mobID)
	-- Early exit
	if not unitID or not mobID or not UnitExists(unitID) or not UnitExists(mobID) then return 3 end

	-- No threat table possible
	if UnitIsFriend(unitID, mobID) or UnitIsPlayer(mobID) then return 3 end

	-- Return aggro status
	local status = select(2, UnitDetailedThreatSituation(unitID, mobID))
	
	-- Wich status do we have?
	if not status then
		return 3
	elseif status >= 2 then
		return 0
	else
		-- Actually needed to do local maxID = select(1, GetMaxThreatUnit(mobID, exclude))
		maxID = mobID.."target"
			
		-- Current unit is in raid (and might be MT)
		if UnitInRaid(maxID) and select(10, GetRaidRosterInfo(maxID)) == "MAINTANK" then
			return 1
		else
			return 2
		end
	end
end
ThreatLine_LuaTexts.GetTankedStatus = GetTankedStatus


-- Much like GetThreatStatusColor(...) but with raid tank support
local function GetTankedColor(unitID, mobID, value)
	-- Query status
	local db 	= PitBull4_ThreatLine.db.profile.global.ThreatLineDB
	local status 	= GetTankedStatus(unitID, mobID)

	-- Enable gradient
	if value and db.use_gradient then
		return 	db.colors[status][0].r * (1-value) + db.colors[status][1].r * value,
				db.colors[status][0].g * (1-value) + db.colors[status][1].g * value,
				db.colors[status][0].b * (1-value) + db.colors[status][1].b * value
	else
		return db.colors[status][1].r, db.colors[status][1].g, db.colors[status][1].b
	end
end
ThreatLine_LuaTexts.GetTankedColor = GetTankedColor


-- Return highest threat holder on mobIDs table (ignoring exclude units)
-- exclude may be player, pet, raidN, raidNpet
local function GetMaxThreatUnit(mobID, exclude)
	-- Early exit #1
	if not UnitExists(mobID) or UnitIsFriend(mobID, "player") or UnitIsPlayer(mobID) then return end

	-- Local variables
	local group, size
	
	-- Get group type
	if GetNumRaidMembers() > 0 then 
		group 	= "raid"
		size 	= GetNumRaidMembers()
	elseif GetNumPartyMembers() > 0 then 
		group 	= "party"
		size 	= GetNumPartyMembers()
	else
		group	= nil
	end
	
	-- Early exit #3
	if not group and not UnitExists("pet") then return end
	
	-- Local variables
	local tempThreat, maxThreat, maxID = 0, 0, nil
	
	-- Add player if not unitID (compare to fixpoint)
	if not exclude or not exclude["player"] then
		tempThreat = select(3, UnitDetailedThreatSituation("player", mobID))
		
		-- Threat querried?
		if tempThreat and tempThreat > maxThreat then
			maxThreat	= tempThreat
			maxID 		= "player"
		end
	end
	
	-- Add pet if not unitID (compare to fixpoint)
	if UnitExists("pet") and (not exclude or not exclude["pet"]) then 
		-- Threat for the pet
		tempThreat	= select(3, UnitDetailedThreatSituation("pet", mobID))

		-- Threat querried?
		if tempThreat and tempThreat > maxThreat then
			maxThreat	= tempThreat
			maxID 		= "pet"
		end
	end

	-- Cycle through Group  (compare to fixpoint)
	if group then
		-- Local variables
		local index, groupID
	
		for index = 1, size do
			-- Threat of this party/raid-member
			groupID 	= group..index
			
			-- Ignore if they are the same
			if not exclude or not exclude[groupID] then
				tempThreat	= select(3, UnitDetailedThreatSituation(groupID, mobID))
				
				-- New highest threat member?
				if tempThreat and tempThreat > maxThreat then 
					maxThreat 	= tempThreat 
					maxID 		= groupID
				end
			end
	
			-- Threat for party/raid-member pet
			groupID 	= groupID.."pet"
			
			-- Ignore if they are the same
			if UnitExists(groupID) and (not exclude or not exclude[groupID]) then
				tempThreat	= select(3, UnitDetailedThreatSituation(groupID, mobID))
				
				-- New highest threat member?
				if tempThreat and tempThreat > maxThreat then 
					maxThreat 	= tempThreat 
					maxID 		= groupID
				end
			end
		end
	end
	
	-- Late exit :P
	return maxID, maxThreat
end
local function GetMaxThreatNonPlayerUnit(mobID)
	return GetMaxThreatUnit(mobID, exclude)
end
ThreatLine_LuaTexts.GetMaxThreatUnit 			= GetMaxThreatUnit
ThreatLine_LuaTexts.GetMaxThreatNonPlayerUnit 	= GetMaxThreatNonPlayerUnit


-- Return unitIDs relative threat to (second) highest player on moIDs threat list
local function GetRelativeThreat(unitID, mobID)
	-- Who to exclude?
	local exclude, found = {}, false
	if UnitIsUnit(unitID, "player") then
		exclude["player"] = true
		found = true
	elseif UnitIsUnit(unitID, "pet") then
		exclude["pet"] = true
		found = true
	elseif UnitIsPlayer(unitID) then
		local group, size
		
		if UnitInRaid(unitID) then
			group 	= "raid"
			size 	= GetNumRaidMembers()
		elseif UnitInParty(unitID) then
			group 	= "party"
			size 	= GetNumPartyMembers()
		else
			group 	= nil
		end
		
		if group then
			local groupID
			
			for index = 1, size do
				-- Threat of this party/raid-member
				groupID 	= group..index
				
				if UnitIsUnit(groupID, unitID) then
					exclude[groupID] = true
					found = true
					break
				end
				
				-- Threat of this party/raid-member
				groupID 	= groupID.."pet"
				
				if UnitIsUnit(groupID, unitID) then
					exclude[groupID] = true
					found = true
					break
				end
			end
		end
	end
	
	-- Relative threat
	local maxID, maxThreat = nil, nil
	if found then
		maxID, maxThreat	= GetMaxThreatUnit(mobID, exclude)
	end
	
	-- Query players threat since we want relative
	local unitThreat = select(3, UnitDetailedThreatSituation(unitID, mobID))

	-- How's first?
	if unitThreat then
		if maxThreat then
			if unitThreat > maxThreat then
				return math.abs(unitThreat - maxThreat), nil
			else
				return math.abs(unitThreat - maxThreat), maxID
			end
		end
		
		return unitThreat, nil
	elseif maxThreat then
		return maxThreat, maxID
	else
		return nil, nil
	end
end
ThreatLine_LuaTexts.GetRelativeThreat = GetRelativeThreat


-- Make sure to return an interesting mob watch threat on
-- UnitID is asumed to be frame.unit for the current frame
local function GetPlayerRelativeID(unitID)
	-- Current (frame.)unit is the player (or empty)
	if not unitID or UnitIsUnit(unitID, "player")then
		return "target"
	end
	
	-- Unit"s target
	local targetID = unitID.."target"

	-- Current (frame.)unit"s target is the player (or empty)
	if  UnitIsUnit(targetID, "player") then
		return "target"
	else
		-- Current (frame.)unit is a mob and (frame.)unit"s target is a/the player
		if not UnitIsPlayer(unitID) and (UnitInRaid(targetID) or UnitInParty(targetID) or UnitIsUnit(targetID, "pet")) then
			return unitID
		-- Current (frame.)unit is a party/raid-member
		elseif UnitInRaid(unitID) or UnitInParty(unitID) or UnitIsUnit(unitID, "pet") then
			return targetID
		end
	end
	
	-- Default return
	return "target"
end
ThreatLine_LuaTexts.GetPlayerRelativeID = GetPlayerRelativeID


-- Smart LuaText relativ threat
-- unitID is assumed to be frame.unit
local function GetPlayerRelativeThreat(unitID)
	-- Relative threat
	local mobID 			= GetPlayerRelativeID(unitID)

	return GetRelativeThreat("player", mobID)
end
ThreatLine_LuaTexts.GetPlayerRelativeThreat = GetPlayerRelativeThreat


-- Smart LuaText aggro color with raidtank support
-- unitID is assumed to be frame.unit
local function GetPlayerTankedColor(unitID, value)
	-- Relative mobID
	local mobID 	= GetPlayerRelativeID(unitID)
	
	-- Return
	return GetTankedColor("player", mobID, value)
end
ThreatLine_LuaTexts.GetPlayerTankedColor = GetPlayerTankedColor


-- Module enabled
function PitBull4_ThreatLine:OnEnable()
	-- Register events
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
	self:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("PLAYER_PET_CHANGED")
	
	-- Force update
	self:PARTY_MEMBERS_CHANGED()
end


-- Module disabled
function PitBull4_ThreatLine:OnDisable()
	-- Register events
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("UNIT_THREAT_LIST_UPDATE")
	self:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	self:UnregisterEvent("RAID_ROSTER_UPDATE")
	self:UnregisterEvent("PLAYER_PET_CHANGED")
	
	-- Force update
	self:PARTY_MEMBERS_CHANGED()
end


-- Need to track thrat information?
function PitBull4_ThreatLine:PARTY_MEMBERS_CHANGED()
	-- In group?
	player_in_group = UnitExists("pet") or GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0
	
	-- Update all frames which are allowed to carry this module
	for classification in pairs(ACCEPTABLE_CLASSIFICATIONS) do
		self:UpdateForClassification(classification)
	end
end
-- Event forwarding
PitBull4_ThreatLine.RAID_ROSTER_UPDATE 				= PitBull4_ThreatLine.PARTY_MEMBERS_CHANGED
PitBull4_ThreatLine.PLAYER_PET_CHANGED 				= PitBull4_ThreatLine.PARTY_MEMBERS_CHANGED


-- Force update
function PitBull4_ThreatLine:PLAYER_TARGET_CHANGED()
	self:UpdateAll()
end
-- Event forwarding
PitBull4_ThreatLine.UNIT_THREAT_LIST_UPDATE 		= PitBull4_ThreatLine.PLAYER_TARGET_CHANGED
PitBull4_ThreatLine.UNIT_THREAT_SITUATION_UPDATE 	= PitBull4_ThreatLine.PLAYER_TARGET_CHANGED


-- Current bar value
function PitBull4_ThreatLine:GetValue(frame)
	-- Early exit
	if not frame or not ACCEPTABLE_CLASSIFICATIONS[frame.classification] or not player_in_group then return nil end
	
	-- Return value
	local threat = select(1, GetPlayerRelativeThreat(frame.unit))
	
	-- Only show if > 0
	if not threat or threat == 0 then
		return  nil
	else
		return math.abs(threat) / 100
	end
end
-- Example bar value
function PitBull4_ThreatLine:GetExampleValue(frame)
	-- Early exit
	if not frame or not ACCEPTABLE_CLASSIFICATIONS[frame.classification] then
		return nil
	end
	
	-- Return an example value
	return 0.25
end


-- Current bar color
function PitBull4_ThreatLine:GetColor(frame, value)
	-- Relative threat unit
	return GetPlayerTankedColor(frame.unit, value)
end
-- Example bar color
function PitBull4_ThreatLine:GetExampleColor(frame, value)
	-- Return an example value
	return self.db.profile.global.ThreatLineDB.colors[0][1]
end


-- Add own color information
PitBull4_ThreatLine:SetColorOptionsFunction(function(self)
	-- Submenu entrys
	return "threatline_0_0", {
		type = "color",
		name = Lmodule["Holding aggro [Min]"],
		desc = Lmodule["Which color to display when having aggro. [Default color or min. gradient if enabled]"],
		get = function(info)
			local db = self.db.profile.global.ThreatLineDB.colors[0][0]
			return db.r, db.g, db.b
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.ThreatLineDB.colors[0][0] = {r=r, g=g, b=b}
			self:UpdateAll()
		end,
	},
	"threatline_0_1", {
		type = "color",
		name = Lmodule["Holding aggro [Max]"],
		desc = Lmodule["Which color to display when having aggro. [Max gradient if enabled]"],
		get = function(info)
			local db = self.db.profile.global.ThreatLineDB.colors[0][1]
			return db.r, db.g, db.b
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.ThreatLineDB.colors[0][1] = {r=r, g=g, b=b}
			self:UpdateAll()
		end,
	},

	"threatline_1_0", {
		type = "color",
		name = Lmodule["Tank aggro [Min]"],
		desc = Lmodule["Which color to display when another tank has aggro. [Default color or min. gradient if enabled]"],
		get = function(info)
			local db = self.db.profile.global.ThreatLineDB.colors[1][0]
			return db.r, db.g, db.b
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.ThreatLineDB.colors[1][0] = {r=r, g=g, b=b}
			self:UpdateAll()
		end,
	},
	"threatline_1_1", {
		type = "color",
		name = Lmodule["Tank aggro [Max]"],
		desc = Lmodule["Which color to display when another tank has aggro. [Max gradient if enabled]"],
		get = function(info)
			local db = self.db.profile.global.ThreatLineDB.colors[1][1]
			return db.r, db.g, db.b
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.ThreatLineDB.colors[1][1] = {r=r, g=g, b=b}
			self:UpdateAll()
		end,
	},
	
	
	"threatline_2_0", {
		type = "color",
		name = Lmodule["Non-tank aggro [Min]"],
		desc = Lmodule["Which color to display when someone else than a tank has aggro. [Default color or min. gradient if enabled]"],
		get = function(info)
			local db = self.db.profile.global.ThreatLineDB.colors[2][0]
			return db.r, db.g, db.b
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.ThreatLineDB.colors[2][0] = {r=r, g=g, b=b}
			self:UpdateAll()
		end,
	},
	"threatline_2_1", {
		type = "color",
		name = Lmodule["Non-tank aggro [Max]"],
		desc = Lmodule["Which color to display when someone else than a tank has aggro. [Max gradient if enabled]"],
		get = function(info)
			local db = self.db.profile.global.ThreatLineDB.colors[2][1]
			return db.r, db.g, db.b
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.ThreatLineDB.colors[2][1] = {r=r, g=g, b=b}
			self:UpdateAll()
		end,
	},
	
	"threatline_3", {
		type = "color",
		name = Lmodule["Undefined aggro"],
		desc = Lmodule["Which color to display when aggro status is unknown."],
		get = function(info)
			local db = self.db.profile.global.ThreatLineDB.colors[3][1]
			return db.r, db.g, db.b
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.ThreatLineDB.colors[3][0] = {r=r, g=g, b=b}
			self.db.profile.global.ThreatLineDB.colors[3][1] = {r=r, g=g, b=b}
			self:UpdateAll()
		end,
	},
	
	"useGradient", {
		type = "toggle",
		name = Lmodule["Enable gradient"],
		desc = Lmodule["Also color by amount of threat. Uses secondary [Max] color for maximum threat difference."],
		get = function(info)
			return self.db.profile.global.ThreatLineDB.use_gradient
		end,
		set = function(info, val)
			self.db.profile.global.ThreatLineDB.use_gradient = val
			self:UpdateAll()
		end,
	},
	
	-- Reset function
	function(info)
		self.db.profile.global.ThreatLineDB = PitBull4.Utils.deep_copy(global_defaults)
	end
end)
